package cs108;

import java.io.PrintStream;
import java.util.List;

public interface TextImage {
    int width();
    int height();
    List<String> drawing();

    static TextImage fromString(String s) {
        return new FromString(s);
    }

    static TextImage filled(int width, int height, char fill) {
        return new Filled(width, height, fill);
    }

    default TextImage flippedHorizontally() {
        return new FlippedHorizontally(this);
    }

    default TextImage transposed() {
        return new Transposed(this);
    }

    default TextImage framed() {
        TextImage tb = fromString('+' + Strings.nCopies(width(), '-') + '+');
        TextImage lr = fromString(Strings.nCopies(height(), '|')).transposed();
        return tb.above(lr.leftOf(this.leftOf(lr)).above(tb));
    }

    default TextImage leftOf(TextImage that) {
        return new LeftOf(this, that);
    }

    default TextImage above(TextImage that) {
        return new Above(this, that);
    }

    default void printOn(PrintStream stream) {
        for (String line : drawing())
            stream.println(line);
    }
}
